"""
OpenAkita Prompt 管线模块

从全文注入改为"编译摘要 + 语义检索 + 预算组装 + 运行时守门"的管线模式。

模块组成:
- compiler.py: 从源 md 编译摘要
- retriever.py: 从 MEMORY.md 检索相关片段
- budget.py: Token 预算裁剪
- builder.py: 组装最终系统提示词
- guard.py: 运行时守门（无工具调用则重试）
"""

from .budget import BudgetConfig, apply_budget
from .builder import build_system_prompt
from .compiler import (
    compile_agent_core,
    compile_agent_tooling,
    compile_all,
    compile_soul,
    compile_user,
)
from .guard import GuardConfig, guard_response
from .retriever import retrieve_memory

__all__ = [
    # Compiler
    "compile_all",
    "compile_soul",
    "compile_agent_core",
    "compile_agent_tooling",
    "compile_user",
    # Retriever
    "retrieve_memory",
    # Budget
    "apply_budget",
    "BudgetConfig",
    # Builder
    "build_system_prompt",
    # Guard
    "guard_response",
    "GuardConfig",
]
